import logging

class CustomStreamLoggingFormatter(logging.Formatter):
    """
    Custom log formatter class to colorize log messages based on their level.
    """

    # Define the color codes
    grey = "\x1b[38;20m"
    green = "\x1b[32;20m"
    yellow = "\x1b[33;20m"
    red = "\x1b[31;20m"
    bold_red = "\x1b[31;1m"
    reset = "\x1b[0m"
    log_format = "%(levelname)s"

    # Define the log message formats for different log levels
    FORMATS = {
        logging.DEBUG: grey + log_format + reset,
        logging.INFO: green + log_format + reset,
        logging.WARNING: yellow + log_format + reset,
        logging.ERROR: red + log_format + reset,
        logging.CRITICAL: bold_red + log_format + reset
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt + " -- %(message)s")
        return formatter.format(record)

class CustomFileLoggingFormatter(logging.Formatter):
    """
    Custom log formatter class for file logging.
    """

    log_format = "%(asctime)s - %(levelname)s - %(message)s"

    def format(self, record):
        formatter = logging.Formatter(self.log_format)
        return formatter.format(record)
    
def get_logger(name=__name__, enable_file_logging=False, file_name="app.log"):
    """
    Returns a logger object configured with a console handler and optionally a file handler.

    Parameters
    ----------
    name : str
        The name of the logger.
    enable_file_logging : bool
        Whether to enable file logging.
    file_name : str
        The name of the log file.
    """
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)  # Set the logger level to DEBUG

    # Console Handler
    console_handler = logging.StreamHandler()
    console_handler.setFormatter(CustomStreamLoggingFormatter())  # Apply the custom formatter
    logger.addHandler(console_handler)

    # Optional File Handler
    if enable_file_logging:
        file_handler = logging.FileHandler(file_name)
        file_handler.setFormatter(CustomFileLoggingFormatter())  # Apply the custom formatter
        logger.addHandler(file_handler)

    logger.propagate = False  # Prevent the logger from propagating to the root logger

    return logger

def set_logger_level(logger, level):
    """
    Set the logger level based on the input string.
    
    Parameters
    ----------
    logger : logging.Logger
        The logger object.
    level : str
        The log level string (DEBUG, INFO, WARNING, ERROR, CRITICAL).
    """
    level = level.upper()
    if level == "DEBUG":
        logger.setLevel(logging.DEBUG)
    elif level == "INFO":
        logger.setLevel(logging.INFO)
    elif level == "WARNING":
        logger.setLevel(logging.WARNING)
    elif level == "ERROR":
        logger.setLevel(logging.ERROR)
    elif level == "CRITICAL":
        logger.setLevel(logging.CRITICAL)
    else:
        logger.warning("Invalid log level. Using default level INFO.")
        logger.setLevel(logging.INFO)

# Create a logger object
logger = get_logger(__name__, enable_file_logging=True, file_name="app.log")


import argparse
import streamlit as st
from streamlit_option_menu import option_menu

from openfactcheck.core.base import OpenFactCheck, OpenFactCheckConfig
from openfactcheck.app.sidebar import sidebar
from openfactcheck.app.evaluate_response import evaluate_response
from openfactcheck.app.evaluate_llm import evaluate_llm
from openfactcheck.app.evaluate_factchecker import evaluate_factchecker
from openfactcheck.app.utils import footer

def parse_args():
    parser = argparse.ArgumentParser(description='Initialize OpenFactCheck with custom configuration.')
    
    # Add arguments here, example:
    parser.add_argument("--config-path", 
                        type=str, 
                        help="Config File Path",
                        default="config.json")
    
    # Parse arguments from command line
    args = parser.parse_args()
    return args
    
class App:
    def __init__(self, config_path: str = "config.json"):
        @st.cache_resource(show_spinner=False)
        def init(config_path: str = "config.json"):
            # Initialize OpenFactCheck
            config = OpenFactCheckConfig(config_path)
            ofc = OpenFactCheck(config)
            return ofc
        
        self.ofc = init(config_path)
        pass

    def run(self):
        # Set up Dashboard
        st.set_page_config(page_title="OpenFactCheck Dashboard", 
                        page_icon=":bar_chart:", 
                        layout="wide",
                        initial_sidebar_state="collapsed")
        
        # Set up footer
        footer("Copyright © 2024 MBZUAI | Made with ❤︎ by OpenFactCheck Team")

        # Set up Sidebar
        sidebar()
                        
        # Title
        st.markdown("<h1 style='text-align: center;'>OpenFactCheck Dashboard</h1>", unsafe_allow_html=True)
        st.markdown("<h5 style='text-align: center;'>An Open-source Factuality Evaluation Demo for LLMs</h5>", unsafe_allow_html=True)

        # Selection Menu
        selected = option_menu(None, ["Evaluate LLM Response", "Evaluate LLM", "Evaluate FactChecker", "Leaderboards", "About"], 
            icons=['card-checklist', 'check-square', "check2-all", "trophy", "info-circle"],
            menu_icon="cast", 
            default_index=0, 
            orientation="horizontal"
        )

        st.info("Please provide OpenAI API Key, Serper API Key and Azure Search Key in the sidebar to evaluate LLM response.")

        # Load the selected page
        if selected == "Evaluate LLM Response":
            evaluate_response(self.ofc)
        elif selected == "Evaluate LLM":
            evaluate_llm(self.ofc)
        elif selected == "Evaluate FactChecker":
            evaluate_factchecker(self.ofc)
        # elif selected == "Leaderboards":
        #     leaderboards()
        # else:
        #     about()


if __name__ == "__main__":
    args = parse_args()

    app = App(args.config_path)
    app.run()